#include <stdio.h>
#include "AmperkaGPRS.h"

GPRS::GPRS(Stream& serial, uint8_t pkPin, uint8_t stPin)
{
  _stPin = stPin;
  _pkPin = pkPin;
  uartDeviceInit(&serial);
}

bool GPRS::begin(void)
{
  if (!uartDeviceCheckWithCMD("AT\r\n","OK\r\n",CMD))
    return false;


  if (!uartDeviceCheckWithCMD("AT+CFUN=1\r\n","OK\r\n",CMD))
    return false;


  if (!checkSIMStatus())
    return false;

  if (!uartDeviceCheckWithCMD("AT+CNMI?\r\n", "+CNMI: 2,2,0,0,0\r\nOK\r\n", CMD)) {
    if (!uartDeviceCheckWithCMD("AT+CNMI=2,2,0,0,0\r\n","OK\r\n", CMD)) {
      return false;
    }
  }

  if (!uartDeviceCheckWithCMD("AT+CMGF?\r\n", "+CMGF: 1\r\nOK\r\n", CMD)) {
    if (!uartDeviceCheckWithCMD("AT+CMGF=1\r\n","OK\r\n", CMD)) {
      return false;
    }
  }

  if (!uartDeviceCheckWithCMD("AT+CLIP=1\r\n","OK\r\n", CMD)) {
      return false;
  }

  if (!uartDeviceCheckWithCMD("AT+COLP=1\r\n","OK\r\n", CMD)) {
      return false;
  }
  return true;
}


// SIM card check
bool GPRS::checkSIMStatus(void) {
    return uartDeviceCheckWithCMD("AT+CPIN?\r\n","+CPIN: READY\r\n\r\nOK\r\n", CMD);
}

// power Up GPRS Shield
void GPRS::powerOn() {
    pinMode(_pkPin, OUTPUT);
    if (!digitalRead(_stPin)) {
        digitalWrite(_pkPin, HIGH);
        delay(3000);
    }
    digitalWrite(_pkPin, LOW);
    delay(3000);
}

// power Off GPRS Shield
void GPRS::powerOff() {
    pinMode(_pkPin, OUTPUT);
    if (digitalRead(_stPin)) {
        digitalWrite(_pkPin, HIGH);
        delay(3000);
    }
    digitalWrite(_pkPin, LOW);
    delay(3000);
}  

bool GPRS::checkPowerOn(void) {
    return uartDeviceCheckWithCMD("AT\r\n","OK\r\n", CMD);
}


// getSignalStrength from SIM900
// returns 0 — 113 dBm or less
//         1 — 111 dBm
//    2...30 — 109... 53 dBm
//        31 — 51 dBm or greater
//        99 — not known or not detectable
int GPRS::getSignal() {
    int i = 0;
    char gprsBuffer[12];
    int signal = 0;
    if (!uartDeviceCheckWithCMD("AT+CSQ\r\n", "+CSQ: ", CMD)) {
        return false;
    }
    uartDeviceCleanBuffer(gprsBuffer, sizeof(gprsBuffer));
    uartDeviceReadBuffer(gprsBuffer, sizeof(gprsBuffer), DEFAULT_TIMEOUT);
    while (gprsBuffer[i]!= ',') {
        i++;
    }
    gprsBuffer[i] = '\0';
    signal = atoi(gprsBuffer);
    if(signal == 0) {
        return -113;
    } else if (signal == 1) {
        return -111;
    } else if (signal >= 2 && signal <= 30) {
        return map(signal, 2 , 30 , -109, -53);
    } else if (signal == 31) {
        return -51;
    } else if (signal == 99) {
        return false;
    }
}

// call someone
// param  number the phone number which you want to call
// returns true on success, false on error
void GPRS::callUp(const char* number) {
	delay(500);
    uartDeviceSendCMD("ATD");
    uartDeviceSendCMD(number);
    uartDeviceSendCMD(";\r\n");
}

// if an incoming call
// param  phone, phone number
// returns true on success, false on error
bool GPRS::incomingCall(char* number) {
    // response is like:
    // +CLIP: "+7XXXXXXXXXX",145,"",,"",0
    if (!uartDeviceWaitForResp("+CLIP: ", CMD))
        return false;

    int i = 0;
    int j = 0;
    char gprsBuffer[80];
    uartDeviceCleanBuffer(gprsBuffer,sizeof(gprsBuffer));
    uartDeviceReadBuffer(gprsBuffer,sizeof(gprsBuffer));

    if (gprsBuffer[i] == '\"') {
        i++;
        while(gprsBuffer[i]!='\"') {
        number[j++] = gprsBuffer[i++];
        }
        number[j] = '\0';
        i++;
    }
    return true;
}

// answer if coming a call   
void GPRS::answerCall() {
    uartDeviceSendCMD("ATA\r\n");
}

// end a call   
void GPRS::endCall() {
    uartDeviceSendCMD("ATH0\r\n");
}

// state call
int GPRS::stateCall() {
    /*  result code:
        0: ready
        2: unknown
        3: ringing
        4: call in progress
    */
    int state;
    char gprsBuffer[10];
    if (!uartDeviceCheckWithCMD("AT+CPAS\r\n","+CPAS: ", CMD)) {
        return -1;
    }
    uartDeviceCleanBuffer(gprsBuffer, 10);
    uartDeviceReadBuffer(gprsBuffer, 10);
    state = gprsBuffer[0] - 48;
    return state;
}

// subscriber answer the call
bool GPRS::subAnswerCall() {
    return uartDeviceWaitForResp("+COLP: ", CMD);
}

// subscriber hung up
bool GPRS::subEndCall() {
    return uartDeviceWaitForResp("NO CARRIER\r\n", CMD);
}

// send text SMS 
// param *number phone number which SMS will be send to
// param  *data   message that will be send to
// returns false on success, true on error
bool GPRS::sendSMS(const char* number, const char* data) {
    char gprsBuffer[32];
    // set message mode to ASCII
    if (!uartDeviceCheckWithCMD("AT+CMGF=1\r\n", "OK\r\n", CMD)) {
        return false;
    }

    uartDeviceSendCMD("AT+CMGS=\"");
    uartDeviceSendCMD(number);

    if (!uartDeviceCheckWithCMD("\"\r\n",">", CMD)) {
        return false;
    }

    uartDeviceSendCMD(data);
    delay(100);
    uartDeviceFlushSerial();
    uartDeviceSendEndMark();
    uartDeviceCleanBuffer(gprsBuffer, sizeof(gprsBuffer));
    uartDeviceReadBuffer(gprsBuffer, sizeof(gprsBuffer), 100);
    if (!strstr(gprsBuffer, "OK\r\n")) {
        return false;
    }
    return true;
}

// if an incoming SMS
bool GPRS::incomingSMS() {
    return uartDeviceWaitForResp("+CMT: ", CMD, 50);
}

// read SMS on buffer
void GPRS::readSMS() {
    char gprsBuffer[80 + 160];
    uartDeviceCleanBuffer(gprsBuffer, sizeof(gprsBuffer));
    uartDeviceReadBuffer(gprsBuffer, sizeof(gprsBuffer), DEFAULT_TIMEOUT);
    Serial.write(gprsBuffer);
}

// read SMS, phone and date if getting a SMS message. It changes SMS status to READ 
// param  message  buffer used to get SMS message
// param  phone  buffer used to get SMS's sender phone number
// param  dataTime  buffer used to get SMS's send dataTime
void GPRS::readSMS(char* message, char* phone, char* dataTime) {
    /* Response is like:
    +CMT: "+79772941911","","15/12/15,01:51:24+12"
    SMS text here
    */

    int i = 0;
    int j = 0;

    char gprsBuffer[80 + 160];

    uartDeviceCleanBuffer(gprsBuffer, sizeof(gprsBuffer));
    uartDeviceReadBuffer(gprsBuffer, sizeof(gprsBuffer), 100);
  
    int len = strlen(gprsBuffer);

    if (gprsBuffer[i]=='\"') {
        i++;
        j = 0;
        while (gprsBuffer[i]!='\"') {
            phone[j++] = gprsBuffer[i++];
        }
        phone[j] = '\0';
        i++;
    }
    if (gprsBuffer[i] == ',')
        i++;
    if (gprsBuffer[i]=='\"') {
        i++;
        while (gprsBuffer[i]!='\"') {
            i++;
        }
        i++;
    }
    if (gprsBuffer[i]==',')
        i++;
    if (gprsBuffer[i]=='\"') {
        i++;
        j = 0;
        while (gprsBuffer[i]!='\"') {
            dataTime[j++] = gprsBuffer[i++];
        }
        dataTime[j] = '\0';
        i++;
    }
    if (gprsBuffer[i]=='\r')
        i++;
    if (gprsBuffer[i]=='\n')
        i++;
    j = 0;
    while (i < len - 2) {
        message[j++] = gprsBuffer[i++];
    }
    message[j] = '\0';
}

// set DateTime for SIM900
// returns false on success, true on error
// format is "yy/MM/dd,hh:mm:ss±zz", where characters
// indicate year (two last digits),month, day, hour, minutes, seconds and time zone 
// E.g. 6th of May 2010, 00:01:52 GMT+2 hours equals to "23/08/16,15:41:52+03"
bool GPRS::setDateTime(const char* dateTime) {
    uartDeviceSendCMD("AT+CCLK=\"");
    uartDeviceSendCMD(dateTime);
    if (!uartDeviceCheckWithCMD("\"\r\n","OK", CMD)) {
        return false;
    }
    return true;
}

// get DateTime from SIM900
// returns false on success, true on error
bool GPRS::getDateTime(char* dateTime) {
    // +CCLK: "14/11/13,21:14:41+04"
    //
    // OK

    int i = 0;
    int j = 0;
    char gprsBuffer[32];
    if (!uartDeviceCheckWithCMD("AT+CCLK?\r\n", "+CCLK: ", CMD)) {
        return false;
    }
    uartDeviceCleanBuffer(gprsBuffer, sizeof(gprsBuffer));
    uartDeviceReadBuffer(gprsBuffer, sizeof(gprsBuffer), DEFAULT_TIMEOUT);
    if (gprsBuffer[i] == '\"') {
        i++;
        while(gprsBuffer[i] != '\"') {
            dateTime[j++] = gprsBuffer[i++];
        }
    }
    dateTime[j] = '\0';
    return true;
}

/*------------------------------GPRS Inet----------------------------------*/
bool GPRS::dataTransferOn(char* apn, char* username, char* password, int timeout) {
    if (!uartDeviceCheckWithCMD("AT+CIPSHUT\r\n","SHUT OK\r\n", CMD)) {
        return false;
    }

    if (!uartDeviceCheckWithCMD("AT+CGATT?\r\n","+CGATT: 1\r\n\r\nOK\r\n", CMD)) {
        if (!uartDeviceCheckWithCMD("AT+CGATT=1\r\n","OK\r\n", CMD)) {
            return false;
        }
        return false;
    }

    if (!uartDeviceCheckWithCMD("AT+CIPMUX=0\r\n","OK\r\n", CMD)) {
        return false;
    }

    uartDeviceSendCMD("AT+CSTT=\"");
    uartDeviceSendCMD(apn);
    uartDeviceSendCMD("\",\"");
    uartDeviceSendCMD(username);
    uartDeviceSendCMD("\",\"");
    uartDeviceSendCMD(password);
    uartDeviceSendCMD("\"\r\n");
    if (!uartDeviceWaitForResp("OK\r\n", CMD)) {
        return false;
    }

    delay(2000);

    if (!uartDeviceCheckWithCMD("AT+CIICR\r\n", "OK\r\n", CMD,  2 * DEFAULT_TIMEOUT)) {
        return false;
    }

    return true;
}

bool GPRS::dataTransferOff() {
    if (!uartDeviceCheckWithCMD("AT+CIPSHUT\r\n","SHUT OK\r\n", CMD)) {
        return false;
    }
    return true;
}

char GPRS::getIPAddress(char* ip) {
    // get local IP address
    // response:
    // AT+CIFSR\r\n
    // \r\n
    // 10.160.57.120\r\n
    // Response error:
    // AT+CIFSR\r\n       
    // \r\n               
    // ERROR\r\n
    char gprsBuffer[32];
    uartDeviceSendCMD("AT+CIFSR");
    uartDeviceSendCMD("\r\n");
    uartDeviceFlushSerial();
    uartDeviceCleanBuffer(gprsBuffer, sizeof(gprsBuffer));
    uartDeviceReadBuffer(gprsBuffer, sizeof(gprsBuffer), DEFAULT_TIMEOUT);
    if (strstr(gprsBuffer, "ERROR")) {
        return false;
    }
    int i = 0;
    char *p;
    p = strstr(gprsBuffer, "\r\n");
    p += 4;
    while (strlen(p) - 2 > 0) {
        ip[i++] = *(p++);
    }
    ip[i] = '\0';

    return true;
}

int GPRS::available() {
    return uartDeviceAvailable();
}

int GPRS::waitAvailable(int time) {
    return uartDeviceWaitAvailable(time);
}

int GPRS::read() {
    return uartDeviceReadSerial();
}

void GPRS::print(const String str) {
    int len = str.length() + 2;
    char request[len];
    str.toCharArray(request, len);
//  Serial.print(request);
    send(request);
}

void GPRS::println(const String str) {
    str += "\r\n";
    int len = str.length() + 2;
    char request[len];
    str.toCharArray(request, len);
//  Serial.print(request);
    send(request);
}

bool GPRS::connect(Protocol ptl, const char* host, int port, int timeout) {
    char strPort[4];
    char gprsBuffer[64];

    if (ptl == TCP) {
        uartDeviceSendCMD("AT+CIPSTART=\"TCP\",\"");
        uartDeviceSendCMD(host);
        uartDeviceSendCMD("\",");
        itoa(port, strPort, 10);
        uartDeviceSendCMD(strPort);
        uartDeviceSendCMD("\r\n");
    } else if (ptl == UDP) {
        uartDeviceSendCMD("AT+CIPSTART=\"UDP\",\"");
        uartDeviceSendCMD(host);
        uartDeviceSendCMD("\",");
        itoa(port, strPort, 10);
        uartDeviceSendCMD(strPort);
        uartDeviceSendCMD("\r\n");
    } else {
        return false;
    }

    if (!uartDeviceWaitForResp("OK\r\n", CMD)) {
        return false;
    }

    if (!uartDeviceWaitForResp("\r\nCONNECT OK\r\n", CMD)) {
        return false;
    }

    return true;
}

bool GPRS::connected() {
    if (!uartDeviceCheckWithCMD("AT+CIPSTATUS\r\n\r\n","\r\nOK\r\n", CMD)) {
        return false;
    }
    if (!uartDeviceWaitForResp("\r\nSTATE: CONNECT OK\r\n", CMD)) {
        return false;
    }
    return true;
}

bool GPRS::disconnect() {
    if (!connected()) {
        return true;
    }
    if (!uartDeviceCheckWithCMD("AT+CIPCLOSE\r\n", "CLOSE OK\r\n", CMD)) {
        return false;
    }
    return true;
}

bool GPRS::startServer(int port) {

}

int GPRS::send(const char * str, int len)
  {
    //char cmd[32];
   char strPort[4];
   if(len > 0){
        //snprintf(cmd,sizeof(cmd),"AT+CIPSEND=%d\r\n",len);
		//sprintf(cmd,"AT+CIPSEND=%d\r\n",len);
    uartDeviceSendCMD("AT+CIPSEND=");
    itoa(len, strPort, 10);
    uartDeviceSendCMD(strPort);
    if(!uartDeviceCheckWithCMD("\r\n",">",CMD)) {
        //if(!uartDeviceCheckWithCMD(cmd,">",CMD)) {
      return 0;
    }
        /*if(0 != uartDeviceCheckWithCMD(str,"SEND OK\r\n", DEFAULT_TIMEOUT * 10 ,DATA)) {
            return 0;
        }*/
            delay(500);
            uartDeviceSendCMD(str);
            delay(500);
            uartDeviceSendEndMark();
            if(!uartDeviceWaitForResp("SEND OK\r\n", DATA, DEFAULT_TIMEOUT * 10, DEFAULT_INTERCHAR_TIMEOUT * 10)) {
              return 0;
            }
          }
          return len;
        }

int GPRS::send(const char * str) {
    //char cmd[32];
          int len=strlen(str);
          char strPort[4];
          if(len > 0){
        //snprintf(cmd,sizeof(cmd),"AT+CIPSEND=%d\r\n",len);
    //sprintf(cmd,"AT+CIPSEND=%d\r\n",len);
            uartDeviceSendCMD("AT+CIPSEND=");
            itoa(len, strPort, 10);
            uartDeviceSendCMD(strPort);
            if(!uartDeviceCheckWithCMD("\r\n",">",CMD)) {
        //if(!uartDeviceCheckWithCMD(cmd,">",CMD)) {
              return 0;
            }
        /*if(0 != uartDeviceCheckWithCMD(str,"SEND OK\r\n", DEFAULT_TIMEOUT * 10 ,DATA)) {
            return 0;
        }*/
            delay(500);
            uartDeviceSendCMD(str);
            delay(500);
            uartDeviceSendEndMark();
            if(!uartDeviceWaitForResp("SEND OK\r\n", DATA, DEFAULT_TIMEOUT * 10, DEFAULT_INTERCHAR_TIMEOUT * 10)) {
              return 0;
            }
          }
          return len;
        }

int GPRS::recv(char* buf, int len)
        {
          uartDeviceCleanBuffer(buf,len);
    uartDeviceReadBuffer(buf,len);   //Ya he llamado a la funcion con la longitud del buffer - 1 y luego le estoy añadiendo el 0
    return strlen(buf);
  }
