#ifndef __AMPERKA_GPRS_H__
#define __AMPERKA_GPRS_H__

#include "uartDevice.h"
#include "Print.h"
#include "Stream.h"

enum Protocol {
    CLOSED = 0,
    TCP    = 1,
    UDP    = 2,
};

class GPRS
{

public:
    GPRS(Stream& serial, uint8_t pkPin = 2, uint8_t stPin = 3); 
    bool begin();
    void powerOn();
    void powerOff();
    bool checkPowerOn();
    int getSignal();
    void callUp(const char* number);
    bool incomingCall(char* number);
    void answerCall();
    void endCall();
    int stateCall();
    bool subAnswerCall();
    bool subEndCall();
    bool sendSMS(const char* number, const char* data);
    bool incomingSMS();
    void readSMS();
    void readSMS(char* message, char* number, char* dateTime);        
    bool setDateTime(const char* dateTime);
    bool getDateTime(char* dateTime);
    bool dataTransferOn(char* apn = 0, char* username = 0, char* password = 0, int timeout = DEFAULT_TIMEOUT);
    bool dataTransferOff();
    char getIPAddress(char* ip);
    int available();
    int waitAvailable(int time);
    int read();
    void print(const String buff);
    void println(const String buff);
    bool connect(Protocol ptl, const char * host, int port, int timeout = 2 * DEFAULT_TIMEOUT);
    bool connected();
    bool disconnect();
    bool startServer(int port);

    int send(const char * str);
    /** send data to socket
     *  @param socket socket
     *  @param str string to be sent
     *  @param len string length
     *  @returns return bytes that actually been send
     */
    int send(const char * str, int len);

    /** read data from socket
     *  @param socket socket
     *  @param buf buffer that will store the data read from socket
     *  @param len string length need to read from socket
     *  @returns bytes that actually read
     */
    int recv(char* buf, int len);

private:
    bool checkSIMStatus(void);
    uint8_t _stPin = 3;
    uint8_t _pkPin = 2;
};
#endif
